#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import unittest

from tests.baseclass import RequiresDevice, RequiresLabeledDevice, RequiresDisk

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class DiskNewUnlabeledTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(_ped.DiskLabelException, _ped.Disk, self._device)


class DiskNewLabeledTestCase(RequiresLabeledDevice):
    def runTest(self):
        result = _ped.Disk(self._device)
        self.assertIsInstance(result, _ped.Disk)
        self.assertEqual(result.type.name, "msdos")


@unittest.skip("Unimplemented test case.")
class DiskGetSetTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskClobberTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskClobberExcludeTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDuplicateTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDestroyTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


class DiskCommitTestCase(RequiresDisk):
    def runTest(self):
        self.assertTrue(self._disk.commit())


class DiskCommitToDevTestCase(RequiresDisk):
    def runTest(self):
        self.assertTrue(self._disk.commit_to_dev())


class DiskCommitToOsTestCase(RequiresDisk):
    def runTest(self):
        self.assertTrue(self._disk.commit_to_os())


class DiskCheckTestCase(RequiresDisk):
    def runTest(self):
        self.assertTrue(self._disk.check())


@unittest.skip("Unimplemented test case.")
class DiskPrintTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


class DiskGetPrimaryPartitionCountTestCase(RequiresDisk):
    def runTest(self):
        # XXX: this could probably test more
        self.assertEqual(self._disk.get_primary_partition_count(), 0)


class DiskGetLastPartitionNumTestCase(RequiresDisk):
    def runTest(self):
        # XXX: this could probably test more
        self.assertEqual(self._disk.get_last_partition_num(), -1)


class DiskGetMaxPrimaryPartitionCountTestCase(RequiresDisk):
    def runTest(self):
        self.assertEqual(self._disk.get_max_primary_partition_count(), 4)


class DiskGetMaxSupportedPartitionCountTestCase(RequiresDisk):
    def runTest(self):
        self.assertEqual(self._disk.get_max_supported_partition_count(), 64)


class DiskGetPartitionAlignmentTestCase(RequiresDisk):
    def runTest(self):
        alignment = self._disk.get_partition_alignment()
        self.assertIsInstance(alignment, _ped.Alignment)
        # These 2 tests assume an MSDOS label as given by RequiresDisk
        self.assertEqual(alignment.offset, 0)
        self.assertEqual(alignment.grain_size, 1)


class DiskMaxPartitionLengthTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.assertEqual(self._disk.max_partition_length(), 4294967295)


class DiskMaxPartitionStartSectorTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.assertEqual(self._disk.max_partition_start_sector(), 4294967295)


class DiskSetFlagTestCase(RequiresDisk):
    def runTest(self):
        # These 2 tests assume an MSDOS label as given by RequiresDisk
        self._disk.set_flag(_ped.DISK_CYLINDER_ALIGNMENT, 1)
        self.assertEqual(self._disk.get_flag(_ped.DISK_CYLINDER_ALIGNMENT), True)
        self._disk.set_flag(_ped.DISK_CYLINDER_ALIGNMENT, 0)
        self.assertEqual(self._disk.get_flag(_ped.DISK_CYLINDER_ALIGNMENT), False)


class DiskGetFlagTestCase(RequiresDisk):
    def runTest(self):
        flag = self._disk.get_flag(_ped.DISK_CYLINDER_ALIGNMENT)
        self.assertIsInstance(flag, bool)


class DiskIsFlagAvailableTestCase(RequiresDisk):
    def runTest(self):
        # We don't know which flags should be available and which shouldn't,
        # but we can at least check that there aren't any tracebacks from
        # trying all of the valid ones.
        for flag in [_ped.DISK_CYLINDER_ALIGNMENT, _ped.DISK_GPT_PMBR_BOOT]:
            self.assertIsInstance(self._disk.is_flag_available(flag), bool)

        # However, an invalid flag should definitely not be available.
        self.assertFalse(self._disk.is_flag_available(1000))


@unittest.skip("Unimplemented test case.")
class DiskAddPartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskRemovePartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDeletePartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDeleteAllTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskSetPartitionGeomTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskMaxmimzePartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetMaxPartitionGeoemtryTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskMinimizeExtendedPartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskNextPartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPartitionTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPartitionBySectorTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


class DiskExtendedPartitionTestCase(RequiresDisk):
    def runTest(self):
        self.assertRaises(_ped.PartitionException, self._disk.extended_partition)


class DiskStrTestCase(RequiresDisk):
    def runTest(self):
        expected = "_ped.Disk instance --\n  dev: %s  type: %s" % (
            repr(self._disk.dev),
            repr(self._disk.type),
        )
        self.assertEqual(expected, str(self._disk))



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import unittest

from tests.baseclass import RequiresDevice

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class CHSGeometryNewTestCase(unittest.TestCase):
    def runTest(self):
        # You're not allowed to create a new CHSGeometry object by hand.
        self.assertRaises(TypeError, _ped.CHSGeometry)


class CHSGeometryGetSetTestCase(RequiresDevice):
    def runTest(self):
        # A device has a CHSGeometry, so we can use that to attempt accessing
        # parameters.
        chs = self._device.hw_geom
        self.assertIsInstance(chs, _ped.CHSGeometry)

        # All attributes are read-only.
        self.assertRaises(AttributeError, setattr, chs, "cylinders", 47)
        self.assertRaises(AttributeError, setattr, chs, "heads", 47)
        self.assertRaises(AttributeError, setattr, chs, "sectors", 47)

        self.assertIsInstance(chs.cylinders, int)
        self.assertIsInstance(chs.heads, int)
        self.assertIsInstance(chs.sectors, int)


class CHSGeometryStrTestCase(RequiresDevice):
    def runTest(self):
        expected = (
            "_ped.CHSGeometry instance --\n  cylinders: %d  heads: %d  sectors: %d"
            % (
                self._device.hw_geom.cylinders,
                self._device.hw_geom.heads,
                self._device.hw_geom.sectors,
            )
        )
        result = str(self._device.hw_geom)
        self.assertEqual(result, expected)



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import unittest

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class FileSystemTypeNewTestCase(unittest.TestCase):
    def runTest(self):
        # You can't create a FileSystemType by hand.
        self.assertRaises(TypeError, _ped.FileSystemType)


class FileSystemTypeGetSetTestCase(unittest.TestCase):
    def runTest(self):
        fstype = _ped.file_system_type_get("ext3")

        self.assertIsInstance(fstype, _ped.FileSystemType)
        self.assertEqual(fstype.name, "ext3")
        self.assertEqual(getattr(fstype, "name"), "ext3")
        self.assertRaises(AttributeError, setattr, fstype, "name", "vfat")
        self.assertRaises(AttributeError, getattr, fstype, "junk")


class FileSystemTypeStrTestCase(unittest.TestCase):
    def runTest(self):
        fstype = _ped.file_system_type_get("ext3")

        self.assertEqual(str(fstype), "_ped.FileSystemType instance --\n  name: ext3")



#
# Test cases for the methods in the parted.partition module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import parted
import unittest
import uuid

from tests.baseclass import RequiresDisk
from tests.baseclass import RequiresGPTDisk

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.


class PartitionNewTestCase(RequiresDisk):
    """
    PartitionNew tests if parted.Partition:
    1) raises user defined exception if called without arguments
    2) if parted.Partition is instantiable with argument fs
    3) if parted.Partition is instantiable without argument fs
    """

    def setUp(self):
        super(PartitionNewTestCase, self).setUp()
        self.geom = parted.Geometry(self.device, start=100, length=100)
        self.fs = parted.FileSystem(type="ext2", geometry=self.geom)
        self.part = parted.Partition(
            self.disk, parted.PARTITION_NORMAL, geometry=self.geom, fs=self.fs
        )

    def reopen(self):
        RequiresDisk.reopen(self)
        self.part = self.disk.partitions[0]

    def runTest(self):
        # Check that not passing args to parted.Partition.__init__ is caught.
        with self.assertRaises((parted.PartitionException,)):
            parted.Partition()

        self.assertIsInstance(self.part, parted.Partition)
        # You don't need to pass a filesystem type at all, since this partition
        # might be FREESPACE or METADATA.
        part_nofs = parted.Partition(
            self.disk, parted.PARTITION_NORMAL, geometry=self.geom
        )
        self.assertIsInstance(part_nofs, parted.Partition)


class PartitionGPTNewTestCase(RequiresGPTDisk):
    """
    Like PartitionNewTestCase but with a GPT-labeled disk image.
    """

    def setUp(self):
        super(PartitionGPTNewTestCase, self).setUp()
        self.geom = parted.Geometry(self.device, start=100, length=100)
        self.fs = parted.FileSystem(type="ext2", geometry=self.geom)
        self.part = parted.Partition(
            self.disk, parted.PARTITION_NORMAL, geometry=self.geom, fs=self.fs
        )

    def reopen(self):
        RequiresDisk.reopen(self)
        self.part = self.disk.partitions[0]

    def runTest(self):
        # Check that not passing args to parted.Partition.__init__ is caught.
        with self.assertRaises((parted.PartitionException,)):
            parted.Partition()

        self.assertIsInstance(self.part, parted.Partition)
        # You don't need to pass a filesystem type at all, since this partition
        # might be FREESPACE or METADATA.
        part_nofs = parted.Partition(
            self.disk, parted.PARTITION_NORMAL, geometry=self.geom
        )
        self.assertIsInstance(part_nofs, parted.Partition)


class PartitionGetSetTestCase(PartitionNewTestCase):
    """
    PartitionGetSet tests "part" instance
    of class parted.Partition(created in baseclass)
    for basic get and set operations on its attributes.
    """

    def runTest(self):
        # Test that passing the kwargs to __init__ works.
        self.assertEqual(self.part.disk, self.disk)
        self.assertIsInstance(self.part.geometry, parted.Geometry)
        self.assertEqual(self.part.type, parted.PARTITION_NORMAL)
        self.assertEqual(self.part.fileSystem.type, "ext2")

        # Test that setting the RW attributes directly works.
        self.part.type = parted.PARTITION_EXTENDED
        self.assertEqual(self.part.type, parted.PARTITION_EXTENDED)

        # Test that setting the RO attributes directly doesn't work.
        exn = (AttributeError, TypeError)
        with self.assertRaises(exn):
            self.part.number = 1
        with self.assertRaises(exn):
            self.part.active = True
        with self.assertRaises(exn):
            self.part.type = "blah"
        with self.assertRaises(exn):
            self.part.disk = self.disk

        # DOS disklabels don't support names
        self.assertEqual(self.part.name, None)
        with self.assertRaises(parted.PartitionException):
            self.part.name = "root"

        # Check that looking for invalid attributes fails properly.
        with self.assertRaises((AttributeError)):
            print(self.part.blah)


class PartitionGetSetGPTTestCase(PartitionGPTNewTestCase):
    """
    Like PartitionGetSetTestCase, but with a GPT-labeled test
    image to work with rather than a DOS-labeled test image.
    """

    def runTest(self):
        # GPT labeled disks can support partition names
        print(self.part.name)
        self.part.name = "root"


class PartitionSetFlagTestCase(PartitionNewTestCase):
    """
    Method setFlag should return True, if flag was set to "on" state.
    Next few flag testcases will inherit from setUp.
    This testcase checks if method setFlag returns:
    1)correct bool when state of BOOT and RAID flags is set to "on"
    on partition.
    2)raises user defined exception on unavailable flag
    See parted/include/parted/disk.in.h for flag numbers.
    Partition flags are dependent on disklabel. In this case msdos label is
    used, see parted library libparted/labels/dos.c for flags availability.
    """

    def setUp(self):
        super(PartitionSetFlagTestCase, self).setUp()
        self.neg_msg = "Method returns unexpected bool value"
        self.flag_set = self.part.setFlag(1)
        self.flag_set1 = self.part.setFlag(5)

    def runTest(self):
        self.assertTrue(self.flag_set, self.neg_msg)
        self.assertTrue(self.flag_set1, self.neg_msg)
        with self.assertRaises((parted.PartitionException,)):
            self.part.setFlag(2)


class PartitionGetFlagTestCase(PartitionSetFlagTestCase):
    """
    Method getFlag should return correct bool value depending on flag
    setting(flag is off=>False; on=>True).
    This testcase checks if method getFlag returns:
    1)correct bools when checks state of BOOT(on), RAID(on) and LVM(off)
    flags.
    """

    def runTest(self):
        self.assertTrue(self.part.getFlag(1), self.neg_msg)
        self.assertTrue(self.part.getFlag(5), self.neg_msg)
        self.assertFalse(self.part.getFlag(6), self.neg_msg)


class PartitionUnsetFlagTestCase(PartitionSetFlagTestCase):
    """
    Method unsetFlag should set flag to "off" state and return True on
    success. PartitionException should be raised on error.
    This testcase checks if method unsetFlag returns:
    1)correct bool value when flag is unset and if flag is in state 'off'
    2)raises user defined exception on unavailable flag
    """

    def runTest(self):
        self.assertTrue(self.part.unsetFlag(1), self.neg_msg)
        self.assertFalse(self.part.getFlag(1), self.neg_msg)

        with self.assertRaises((parted.PartitionException,)):
            self.part.unsetFlag(2)


class PartitionIsFlagAvailableTestCase(PartitionNewTestCase):
    """
    Method isFlagAvailable should return bool value whenever flag is
    available according to chosen disk label setting and disk proportions
    itself.
    This testcase checks if method isFlagAvailable returns:
    1)bool value without traceback
    2)false on nonexistent flag
    3)raises user defined exception when called on inactive partition
    """

    def runTest(self):
        for f in [
            "PARTITION_BOOT",
            "PARTITION_ROOT",
            "PARTITION_SWAP",
            "PARTITION_HIDDEN",
            "PARTITION_RAID",
            "PARTITION_LVM",
            "PARTITION_LBA",
            "PARTITION_HPSERVICE",
            "PARTITION_PALO",
            "PARTITION_PREP",
            "PARTITION_MSFT_RESERVED",
            "PARTITION_APPLE_TV_RECOVERY",
            "PARTITION_BIOS_GRUB",
            "PARTITION_DIAG",
            "PARTITION_MSFT_DATA",
            "PARTITION_IRST",
            "PARTITION_ESP",
            "PARTITION_NONFS",
            "PARTITION_CHROMEOS_KERNEL",
            "PARTITION_BLS_BOOT",
            "PARTITION_LINUX_HOME",
            "PARTITION_NO_AUTOMOUNT",
        ]:
            if not hasattr(parted, f):
                continue
            attr = getattr(parted, f)
            self.assertIsInstance(self.part.isFlagAvailable(attr), bool)

        self.assertFalse(self.part.isFlagAvailable(1000))

        with self.assertRaises((parted.PartitionException,)):
            self.part = parted.Partition(
                self.disk, parted.PARTITION_FREESPACE, geometry=self.geom
            )
            self.part.isFlagAvailable(parted.PARTITION_BOOT)


class PartitionGetFlagsAsStringTestCase(PartitionSetFlagTestCase):
    """
    Method getFlagsAsString should return all flags which are in state "on"
    as comma separated list.
    """

    def runTest(self):
        self.assertEqual(self.part.getFlagsAsString(), "boot, raid")


@unittest.skipUnless(
    hasattr(parted, "DISK_TYPE_PARTITION_TYPE_ID"), "requires parted >= 3.5"
)
class PartitionMSDOSTypeIDTestCase(PartitionNewTestCase):
    def runTest(self):
        # The DOS disklabel should support ID
        self.assertTrue(self.disk.supportsFeature(parted.DISK_TYPE_PARTITION_TYPE_ID))

        # Check the default ID
        self.assertEqual(self.part.type_id, 0x83)

        # Update and check a new ID
        self.part.type_id = 0xA7
        self.assertEqual(self.part.type_id, 0xA7)

        # Persist the changes
        self.disk.addPartition(self.part)
        self.disk.commitToDevice()

        # Check update was persistent
        self.reopen()
        self.assertEqual(self.part.type_id, 0xA7)


@unittest.skipUnless(
    hasattr(parted, "DISK_TYPE_PARTITION_TYPE_ID"), "requires parted >= 3.5"
)
class PartitionGPTTypeIDTestCase(PartitionGPTNewTestCase):
    def runTest(self):
        # The GPT disklabel should not support ID
        self.assertFalse(self.disk.supportsFeature(parted.DISK_TYPE_PARTITION_TYPE_ID))
        self.assertEqual(self.part.type_id, 0)


@unittest.skipUnless(
    hasattr(parted, "DISK_TYPE_PARTITION_TYPE_UUID"), "requires parted >= 3.5"
)
class PartitionMSDOSTypeUUIDTestCase(PartitionNewTestCase):
    def runTest(self):
        # The DOS disklabel should NOT support UUID
        self.assertFalse(
            self.disk.supportsFeature(parted.DISK_TYPE_PARTITION_TYPE_UUID)
        )
        self.assertIsNone(self.part.type_uuid)


@unittest.skipUnless(
    hasattr(parted, "DISK_TYPE_PARTITION_TYPE_UUID"), "requires parted >= 3.5"
)
class PartitionGPTTypeUUIDTestCase(PartitionGPTNewTestCase):
    def runTest(self):
        # The GPT disklabel should not support UUID
        self.assertTrue(self.disk.supportsFeature(parted.DISK_TYPE_PARTITION_TYPE_UUID))

        # Check the default UUID
        default = uuid.UUID("0fc63daf-8483-4772-8e79-3d69d8477de4")
        self.assertEqual(self.part.type_uuid, default.bytes)

        # Update and check a new UUID
        ia64root = uuid.UUID("993d8d3d-f80e-4225-855a-9daf8ed7ea97")
        self.part.type_uuid = ia64root.bytes
        self.assertEqual(self.part.type_uuid, ia64root.bytes)

        # Persist the changes
        self.disk.addPartition(self.part)
        self.disk.commitToDevice()

        # Check update was persistent
        self.reopen()
        self.assertEqual(self.part.type_uuid, ia64root.bytes)


@unittest.skip("Unimplemented test case.")
class PartitionGetMaxGeometryTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class PartitionNextPartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class PartitionGetSizeTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class PartitionGetLengthTestCase(RequiresDisk):
    def runTest(self):
        length = 100
        geom = parted.Geometry(self.device, start=100, length=length)
        part = parted.Partition(self.disk, parted.PARTITION_NORMAL, geometry=geom)
        constraint = parted.Constraint(exactGeom=geom)
        self.disk.addPartition(part, constraint)
        self.disk.commit()
        part = self.disk.partitions[0]

        self.assertEqual(part.getLength(), part.geometry.length)
        self.assertEqual(part.getLength(), length)


@unittest.skip("Unimplemented test case.")
class PartitionGetMaxAvailableSizeTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class PartitionGetDeviceNodeNameTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class PartitionGetPedPartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class PartitionStrTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")



#
# Test cases for the methods in the parted.geometry module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import parted
import unittest

from tests.baseclass import RequiresDevice

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
@unittest.skip("Unimplemented test case.")
class GeometryNewTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryGetSetTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryCheckTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryContainsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryContainsSectorTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryEqualTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryGetSizeTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class GeometryGetLengthTestCase(RequiresDevice):
    def runTest(self):
        length = 137
        geom = parted.Geometry(self.device, start=100, length=length)

        self.assertEqual(geom.getLength(), geom.length)
        self.assertEqual(geom.getLength(), length)


@unittest.skip("Unimplemented test case.")
class GeometryIntersectTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryMapTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryOverlapsWithTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryReadTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometrySyncTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class GeometryWriteTestCase(RequiresDevice):
    def setUp(self):
        super().setUp()
        self.geom = parted.Geometry(self.device, start=10, length=100)

    def runTest(self):
        self._device.open()
        self.assertTrue(
            self.geom.write("".join(["\x00"] * self.device.sectorSize), 0, 1)
        )
        self.assertTrue(
            self.geom.write("".join(["\x01"] * self.device.sectorSize), 0, 1)
        )
        self._device.close()


@unittest.skip("Unimplemented test case.")
class GeometryGetPedGeometryTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class GeometryStrTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")



#!/usr/bin/python3

import sys

from pocketlint import PocketLintConfig, PocketLinter


class PypartedLintConfig(PocketLintConfig):
    @property
    def extraArgs(self):
        return ["--extension-pkg-whitelist", "_ped"]


if __name__ == "__main__":
    conf = PypartedLintConfig()
    linter = PocketLinter(conf)
    rc = linter.run()
    sys.exit(rc)



#
# Test cases for the methods in the parted.device module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import unittest
from tests.baseclass import RequiresDevice

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
@unittest.skip("Unimplemented test case.")
class DeviceNewTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceGetSetTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceOpenTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceCloseTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceDestroyTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceRemoveFromCacheTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceBeginExternalAccessTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceEndExternalAccessTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceReadTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceWriteTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceSyncTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceCheckTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceStartSectorToCylinderTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceEndSectorToCylinderTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceStartCylinderToSectorTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceEndCylinderToSectorTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceGetSizeTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class DeviceGetLengthTestCase(RequiresDevice):
    def runTest(self):
        self.assertEqual(self.device.getLength(), self.device.length)


@unittest.skip("Unimplemented test case.")
class DeviceGetSizeAsSectorsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceGetConstraintTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceGetPedDeviceTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceStrTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import sys
import _ped
from tests.baseclass import RequiresFileSystem

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class FileSystemNewTestCase(RequiresFileSystem):
    def runTest(self):
        fstype = _ped.file_system_type_get("ext2")

        with self.assertRaises(TypeError):
            _ped.FileSystem(type=None, geom=None)
            _ped.FileSystem(type=fstype, geom=None)

        fs = _ped.FileSystem(type=fstype, geom=self._geometry)
        self.assertIsInstance(fs, _ped.FileSystem)


class FileSystemGetSetTestCase(RequiresFileSystem):
    def runTest(self):
        fstype = _ped.file_system_type_get("ext2")
        fs = _ped.FileSystem(type=fstype, geom=self._geometry)

        self.assertIsInstance(fs, _ped.FileSystem)
        self.assertEqual(fs.type, fstype)
        self.assertEqual(getattr(fs, "type"), fstype)
        # read-only attribute
        if sys.version_info[0] == 3:
            self.assertRaises(AttributeError, setattr, fs, "type", fstype)
        else:
            self.assertRaises(TypeError, setattr, fs, "type", fstype)
        self.assertRaises(AttributeError, getattr, fs, "junk")


class FileSystemStrTestCase(RequiresFileSystem):
    def runTest(self):
        fstype = _ped.file_system_type_get("ext2")
        fs = _ped.FileSystem(type=fstype, geom=self._geometry)
        # don't use assertEqual b/c __str__ includes memory addresses of
        # fstype and geom objects. This is easier.
        self.assertTrue(str(fs).startswith("_ped.FileSystem instance --"))



#
# Test cases for the methods in the parted module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

from __future__ import division

import _ped
import parted
import unittest
from tests.baseclass import RequiresDevice, RequiresDeviceNode

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class FormatBytesTestCase(unittest.TestCase):
    def runTest(self):
        self.assertRaises(SyntaxError, parted.formatBytes, 57, "GIB")
        self.assertEqual(1e-24, parted.formatBytes(1, "YB"))
        self.assertEqual(1 / 2**80, parted.formatBytes(1, "YiB"))
        self.assertEqual(1, parted.formatBytes(1, "B"))
        self.assertEqual(1, parted.formatBytes(1e24, "YB"))
        self.assertEqual(1, parted.formatBytes(2**80, "YiB"))


class BytesToSectorsTestCase(unittest.TestCase):
    def runTest(self):
        self.assertRaises(SyntaxError, parted.sizeToSectors, 9, "yb", 1)
        self.assertEqual(
            int(parted.sizeToSectors(7777.0, "B", 512)),
            parted.sizeToSectors(7777.0, "B", 512),
        )
        self.assertEqual(parted.sizeToSectors(1000, "B", 512), 2)


class GetLabelsTestCase(unittest.TestCase):
    def runTest(self):
        self.assertGreater(len(parted.getLabels()), 0)
        self.assertSetEqual(parted.getLabels("ppcc"), set())
        self.assertSetEqual(parted.getLabels("sparc6"), set())
        self.assertSetEqual(parted.getLabels("i586"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("s390"), {"dasd", "msdos"})
        self.assertSetEqual(parted.getLabels("s390x"), {"dasd", "msdos"})
        self.assertSetEqual(parted.getLabels("sparc"), {"sun"})
        self.assertSetEqual(parted.getLabels("sparc64"), {"sun"})
        self.assertSetEqual(parted.getLabels("ppc"), {"amiga", "gpt", "mac", "msdos"})
        self.assertSetEqual(parted.getLabels("ppc64"), {"amiga", "gpt", "mac", "msdos"})
        self.assertSetEqual(parted.getLabels("ppc64le"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("alpha"), {"bsd", "msdos"})
        self.assertSetEqual(parted.getLabels("ia64"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("aarch64"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("armv7l"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("riscv32"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("riscv64"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("loongarch32"), {"gpt", "msdos"})
        self.assertSetEqual(parted.getLabels("loongarch64"), {"gpt", "msdos"})


class GetDeviceTestCase(RequiresDeviceNode):
    def runTest(self):
        # Check that a DiskException is raised for an invalid path
        self.assertRaises(parted.DeviceException, parted.getDevice, None)
        self.assertRaises(parted.IOException, parted.getDevice, "")
        self.assertRaises(parted.IOException, parted.getDevice, "/dev/whatever")

        # Check that we get a parted.Device back
        self.assertIsInstance(parted.getDevice(self.path), parted.Device)

        # Make sure the device node paths match
        self.assertEqual(parted.getDevice(self.path).path, self.path)


class GetAllDevicesTestCase(unittest.TestCase):
    def setUp(self):
        self.devices = parted.getAllDevices()

    def runTest(self):
        # Check self.devices and see that it's a list
        self.assertEqual(type(self.devices).__name__, "list")

        # And make sure each element of the list is a parted.Device
        for dev in self.devices:
            self.assertIsInstance(dev, parted.Device)


@unittest.skip("Unimplemented test case.")
class ProbeForSpecificFileSystemTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class ProbeFileSystemTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class FreshDiskTestCase(RequiresDevice):
    def runTest(self):
        # Make sure we get SyntaxError when using an invalid disk type
        self.assertRaises(KeyError, parted.freshDisk, self.device, "cheese")
        self.assertRaises(KeyError, parted.freshDisk, self.device, "crackers")

        # Create a new disk for each disk type key, verify each one
        # XXX: Skip over dvh for now (SGI disk label), which doesn't seem to have
        # working libparted support.  If anyone with an SGI cares, patches welcome.
        for key in parted.diskType.keys():
            if key in ["dvh", "aix"]:
                continue
            disk = parted.freshDisk(self.device, key)
            self.assertIsInstance(disk, parted.Disk)
            self.assertEqual(disk.type, key)

        # Create a new disk each disk type value, verify each one
        for value in parted.diskType.values():
            if value.name in ["dvh", "aix"]:
                continue
            disk = parted.freshDisk(self.device, value)
            self.assertIsInstance(disk, parted.Disk)
            self.assertEqual(parted.diskType[disk.type], value)


@unittest.skip("Unimplemented test case.")
class IsAlignToCylindersTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class ToggleAlignToCylindersTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class VersionTestCase(unittest.TestCase):
    def runTest(self):
        ver = parted.version()
        self.assertEqual(ver["libparted"], _ped.libparted_version())
        self.assertEqual(ver["pyparted"], _ped.pyparted_version())



#
# Test cases for the methods in the parted.disk module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import parted
import unittest

from tests.baseclass import RequiresDisk

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
@unittest.skip("Unimplemented test case.")
class DiskNewTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetSetTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskClobberTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDuplicateTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDestroyTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskCommitTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskCommitToDeviceTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskCommitToOSTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskCheckTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskSupportsFeatureTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class DiskAddPartitionTestCase(RequiresDisk):
    """
    addPartition should return True if partition is added successfully(even
    without committing)
    """

    def runTest(self):
        self.disk.setFlag(parted.DISK_CYLINDER_ALIGNMENT)

        length = 100
        geom = parted.Geometry(self.device, start=100, length=length)
        part = parted.Partition(self.disk, parted.PARTITION_NORMAL, geometry=geom)
        constraint = parted.Constraint(exactGeom=geom)
        self.assertTrue(self.disk.addPartition(part, constraint))


@unittest.skip("Unimplemented test case.")
class DiskRemovePartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDeletePartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskDeleteAllPartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskSetPartitionGeometryTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskMaximizePartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskCalculateMaxPartitionGeometryTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskMinimizeExtendedPartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPartitionBySectorTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class DiskGetMaxSupportedPartitionCountTestCase(RequiresDisk):
    """
    maxSupportedPartitionCount should return value 64, based on default
    value MAX_NUM_PARTS(parted/libparted/arch/linux.c) applied if it cannot
    find value in /sys/block/DEV/ext_range (RequiresDisk implies there is
    no ext_range value). Also see testcase
    DiskGetMaxSupportedPartitionCountTestCase in tests/test__ped_disk ,
    which tests value returned by source C function defined in module _ped
    """

    def runTest(self):
        self.assertEqual(self.disk.maxSupportedPartitionCount, 64)


class DiskMaxPartitionLengthTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.assertEqual(self.disk.maxPartitionLength, 4294967295)


class DiskMaxPartitionStartSectorTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.assertEqual(self.disk.maxPartitionStartSector, 4294967295)


class DiskGetFlagTestCase(RequiresDisk):
    def runTest(self):
        flag = self.disk.getFlag(parted.DISK_CYLINDER_ALIGNMENT)
        self.assertIsInstance(flag, bool)


class DiskSetFlagTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.disk.setFlag(parted.DISK_CYLINDER_ALIGNMENT)
        flag = self.disk.getFlag(parted.DISK_CYLINDER_ALIGNMENT)
        self.assertEqual(flag, True)


class DiskUnsetFlagTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        self.disk.unsetFlag(parted.DISK_CYLINDER_ALIGNMENT)
        flag = self.disk.getFlag(parted.DISK_CYLINDER_ALIGNMENT)
        self.assertEqual(flag, False)


class DiskIsFlagAvailableTestCase(RequiresDisk):
    def runTest(self):
        # This test assumes an MSDOS label as given by RequiresDisk
        available = self.disk.isFlagAvailable(parted.DISK_CYLINDER_ALIGNMENT)
        self.assertEqual(available, True)


@unittest.skip("Unimplemented test case.")
class DiskGetExtendedPartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetLogicalPartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPrimaryPartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetRaidPartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetLVMPartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetFreeSpaceRegionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetFreeSpacePartitionsTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetFirstPartitionTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPartitionByPathTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskGetPedDiskTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DiskStrTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import os
import _ped
import tempfile
import unittest
import uuid

from tests.baseclass import RequiresDisk, RequiresPartition, RequiresGPTPartition

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class PartitionNewTestCase(RequiresDisk):
    def runTest(self):
        # Check that not passing args to _ped.Partition.__init__ is caught.
        self.assertRaises(TypeError, _ped.Partition)

        # Or passing the arguments in the wrong order.
        self.assertRaises(
            TypeError,
            _ped.Partition,
            _ped.file_system_type_get("ext2"),
            _ped.PARTITION_NORMAL,
            self._disk,
            0,
            100,
        )

        part = _ped.Partition(
            self._disk, _ped.PARTITION_NORMAL, 0, 100, _ped.file_system_type_get("ext2")
        )
        self.assertIsInstance(part, _ped.Partition)

        # You don't need to pass a filesystem type at all, since this partition
        # might be FREESPACE or METADATA.
        part = _ped.Partition(self._disk, _ped.PARTITION_NORMAL, 0, 100)
        self.assertIsInstance(part, _ped.Partition)


class PartitionGetSetTestCase(RequiresPartition):
    def runTest(self):
        # Test that passing the kwargs to __init__ works.
        self.assertEqual(self._part.disk, self._disk)
        self.assertIsInstance(self._part.geom, _ped.Geometry)
        self.assertEqual(self._part.type, _ped.PARTITION_NORMAL)
        self.assertEqual(self._part.fs_type.name, "ext2")

        # Test that setting the RW attributes directly works.
        self._part.type = _ped.PARTITION_EXTENDED
        self.assertEqual(getattr(self._part, "type"), _ped.PARTITION_EXTENDED)

        # Test that setting the RO attributes directly doesn't work.
        exn = (AttributeError, TypeError)
        self.assertRaises(exn, setattr, self._part, "num", 1)
        self.assertRaises(
            exn, setattr, self._part, "fs_type", _ped.file_system_type_get("fat32")
        )
        self.assertRaises(
            exn, setattr, self._part, "geom", _ped.Geometry(self._device, 10, 20)
        )
        self.assertRaises(exn, setattr, self._part, "disk", self._disk)

        # Check that values have the right type.
        self.assertRaises(exn, setattr, self._part, "type", "blah")

        # Check that looking for invalid attributes fails properly.
        self.assertRaises(AttributeError, getattr, self._part, "blah")


class PartitionDestroyTestCase(RequiresPartition):
    def runTest(self):
        self.assertEqual(self._part.destroy(), None)


class PartitionIsActiveTestCase(RequiresPartition):
    def runTest(self):
        # A partition is active as long as it's not METADATA or FREE.
        for ty in [
            _ped.PARTITION_NORMAL,
            _ped.PARTITION_LOGICAL,
            _ped.PARTITION_EXTENDED,
            _ped.PARTITION_PROTECTED,
        ]:
            self._part.type = ty
            self.assertTrue(self._part.is_active())

        for ty in [_ped.PARTITION_FREESPACE, _ped.PARTITION_METADATA]:
            # Can't have a partition of these two types that also has a
            # filesystem type associated with it.  libparted doesn't like
            # that combination.
            self._part = _ped.Partition(self._disk, ty, 0, 100)
            self.assertFalse(self._part.is_active())


class PartitionSetFlagTestCase(RequiresPartition):
    def runTest(self):
        self.assertTrue(self._part.set_flag(_ped.PARTITION_BOOT, 1))
        # try setting unavailable flag
        with self.assertRaises(_ped.PartedException):
            self._part.set_flag(1000, 1)


class PartitionGetFlagTestCase(RequiresPartition):
    def runTest(self):
        self.assertTrue(self._part.set_flag(_ped.PARTITION_BOOT, 1))
        self.assertTrue(self._part.get_flag(_ped.PARTITION_BOOT))

        # try getting unavailable flag - doesn't raise an exception


#        with self.assertRaises(_ped.PartedException):
#            self._part.get_flag(1000)


class PartitionIsFlagAvailableTestCase(RequiresPartition):
    def runTest(self):
        # We don't know which flags should be available and which shouldn't,
        # but we can at least check that there aren't any tracebacks from
        # trying all of the valid ones.
        for f in [
            "PARTITION_BOOT",
            "PARTITION_ROOT",
            "PARTITION_SWAP",
            "PARTITION_HIDDEN",
            "PARTITION_RAID",
            "PARTITION_LVM",
            "PARTITION_LBA",
            "PARTITION_HPSERVICE",
            "PARTITION_PALO",
            "PARTITION_PREP",
            "PARTITION_MSFT_RESERVED",
            "PARTITION_APPLE_TV_RECOVERY",
            "PARTITION_BIOS_GRUB",
            "PARTITION_DIAG",
            "PARTITION_MSFT_DATA",
            "PARTITION_IRST",
            "PARTITION_ESP",
            "PARTITION_NONFS",
            "PARTITION_CHROMEOS_KERNEL",
            "PARTITION_BLS_BOOT",
            "PARTITION_LINUX_HOME",
            "PARTITION_NO_AUTOMOUNT",
        ]:
            if not hasattr(_ped, f):
                continue
            attr = getattr(_ped, f)
            self.assertIsInstance(self._part.is_flag_available(attr), bool)

        # However, an invalid flag should definitely not be available.
        self.assertFalse(self._part.is_flag_available(1000))

        # Partitions that are inactive should not have any available flags.
        self._part = _ped.Partition(self._disk, _ped.PARTITION_FREESPACE, 0, 100)
        self.assertRaises(
            _ped.PartitionException, self._part.is_flag_available, _ped.PARTITION_BOOT
        )


class PartitionSetSystemTestCase(RequiresPartition):
    def runTest(self):
        self.assertTrue(self._part.set_system(_ped.file_system_type_get("fat32")))

        self.assertRaises(TypeError, self._part.set_system, 47)

        # Partitions that are inactive cannot have the system type set.
        self._part = _ped.Partition(self._disk, _ped.PARTITION_FREESPACE, 0, 100)
        self.assertRaises(
            _ped.PartitionException,
            self._part.set_system,
            _ped.file_system_type_get("ext2"),
        )


class PartitionSetNameTestCase(RequiresPartition):
    def runTest(self):
        # The DOS disklabel does not support naming.
        self.assertRaises(_ped.PartitionException, self._part.set_name, "blah")

        # These should work.
        self._disk = _ped.disk_new_fresh(self._device, _ped.disk_type_get("mac"))
        self._part = _ped.Partition(
            self._disk,
            _ped.PARTITION_NORMAL,
            0,
            100,
            _ped.file_system_type_get("fat32"),
        )
        self.assertTrue(self._part.set_name("blah"))
        self.assertEqual(self._part.get_name(), "blah")

        # Partitions that are inactive won't work.
        self._part = _ped.Partition(self._disk, _ped.PARTITION_FREESPACE, 0, 100)
        self.assertRaises(_ped.PartitionException, self._part.get_name)


class PartitionGetNameTestCase(RequiresPartition):
    def runTest(self):
        # The DOS disklabel does not support naming.
        self.assertRaises(_ped.PartitionException, self._part.get_name)

        # Partitions that are inactive won't work either.
        self._part = _ped.Partition(self._disk, _ped.PARTITION_FREESPACE, 0, 100)
        self.assertRaises(_ped.PartitionException, self._part.get_name)

        # Mac disk labels do support naming, but there still has to be a name.
        self._disk = _ped.disk_new_fresh(self._device, _ped.disk_type_get("mac"))
        self._part = _ped.Partition(
            self._disk,
            _ped.PARTITION_NORMAL,
            0,
            100,
            _ped.file_system_type_get("fat32"),
        )
        self.assertEqual(self._part.get_name(), "untitled")

        # Finally, Mac disk labels with a name will work.
        self._part.set_name("blah")
        self.assertEqual(self._part.get_name(), "blah")


@unittest.skipUnless(
    hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_ID"), "requires parted >= 3.5"
)
class PartitionMSDOSTypeIDTestCase(RequiresPartition):
    def runTest(self):
        # The DOS disklabel should support ID
        self.assertTrue(self._disk.type.check_feature(_ped.DISK_TYPE_PARTITION_TYPE_ID))

        # Check the default ID
        self.assertEqual(self._part.get_type_id(), 0x83)

        # Update and check a new ID
        self._part.set_type_id(0xA7)
        self.assertEqual(self._part.get_type_id(), 0xA7)

        # Persist the changes
        self._disk.add_partition(
            self._part, self._device.get_optimal_aligned_constraint()
        )
        self._disk.commit_to_dev()

        # Check update was persistent
        self.reopen()
        self.assertEqual(self._part.get_type_id(), 0xA7)


@unittest.skipUnless(
    hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_ID"), "requires parted >= 3.5"
)
class PartitionGPTTypeIDTestCase(RequiresGPTPartition):
    def runTest(self):
        # The GPT disklabel should not support ID
        self.assertFalse(
            self._disk.type.check_feature(_ped.DISK_TYPE_PARTITION_TYPE_ID)
        )
        self.assertEqual(self._part.get_type_id(), 0)


@unittest.skipUnless(
    hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_UUID"), "requires parted >= 3.5"
)
class PartitionMSDOSTypeUUIDTestCase(RequiresPartition):
    def runTest(self):
        # The DOS disklabel should NOT support UUID
        self.assertFalse(
            self._disk.type.check_feature(_ped.DISK_TYPE_PARTITION_TYPE_UUID)
        )
        self.assertRaises(_ped.PartitionException, self._part.get_type_uuid)


@unittest.skipUnless(
    hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_UUID"), "requires parted >= 3.5"
)
class PartitionGPTTypeUUIDTestCase(RequiresGPTPartition):
    def runTest(self):
        # The GPT disklabel should not support UUID
        self.assertTrue(
            self._disk.type.check_feature(_ped.DISK_TYPE_PARTITION_TYPE_UUID)
        )

        # Check the default UUID
        default = uuid.UUID("0fc63daf-8483-4772-8e79-3d69d8477de4")
        self.assertEqual(self._part.get_type_uuid(), default.bytes)

        # Update and check a new UUID
        ia64root = uuid.UUID("993d8d3d-f80e-4225-855a-9daf8ed7ea97")
        self._part.set_type_uuid(ia64root.bytes)
        self.assertEqual(self._part.get_type_uuid(), ia64root.bytes)

        # Persist the changes
        self._disk.add_partition(self._part)
        self._disk.commit_to_dev()

        # Check update was persistent
        self.reopen()
        self.assertEqual(self._part.get_type_uuid(), ia64root.bytes)


class PartitionIsBusyTestCase(RequiresPartition):
    def setUp(self):
        super().setUp()
        self.addCleanup(self.removeMountpoint)
        self.mountpoint = None

    def doMount(self):
        self.mountpoint = tempfile.mkdtemp()
        os.system("mount -o loop %s %s" % (self.path, self.mountpoint))

    def mkfs(self):
        os.system("mkfs.ext2 -F -q %s" % self.path)

    def removeMountpoint(self):
        if self.mountpoint:
            os.system("umount %s" % self.mountpoint)
            os.rmdir(self.mountpoint)

    def runTest(self):
        # partitions aren't busy until they're mounted.
        self.assertFalse(self._part.is_busy())


# TODO:  need to figure out how to make a loopback device look mounted to
# libparted
#        self.mkfs()
#        self.doMount()
#        self.assertTrue(self._part.is_busy())


class PartitionGetPathTestCase(RequiresPartition):
    def runTest(self):
        self.assertNotEqual(self._part.get_path(), "")


class PartitionStrTestCase(RequiresPartition):
    def runTest(self):
        self.assertTrue(str(self._part).startswith("_ped.Partition instance"))



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import unittest

from tests.baseclass import RequiresDevice

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class DeviceNewTestCase(unittest.TestCase):
    def runTest(self):
        # You're not allowed to create a new Device object by hand.
        self.assertRaises(TypeError, _ped.Device)


class DeviceGetSetTestCase(RequiresDevice):
    def runTest(self):
        # All attributes should be readable, but none should be writeable.
        for attr in [
            "model",
            "path",
            "type",
            "sector_size",
            "phys_sector_size",
            "length",
            "open_count",
            "read_only",
            "external_mode",
            "dirty",
            "boot_dirty",
            "host",
            "did",
        ]:
            self.assertNotEqual(getattr(self._device, attr), None)
            self.assertRaises(AttributeError, setattr, self._device, attr, 47)


class DeviceIsBusyTestCase(RequiresDevice):
    def runTest(self):
        # Devices aren't busy until they're mounted.
        self.assertFalse(self._device.is_busy())


# TODO:  need to figure out how to make a loopback device look mounted to
# libparted
#        self.mkfs()
#        self.doMount()
#        self.assertTrue(self._device.is_busy())


class DeviceOpenTestCase(RequiresDevice):
    def runTest(self):
        self.assertTrue(self._device.open())
        self.assertEqual(self._device.open_count, 1)
        self._device.close()

        # Not allowed to open a device that's already been opened for external
        # access, so test that now.
        self._device.begin_external_access()
        self.assertRaises(_ped.IOException, self._device.open)
        self._device.end_external_access()
        self.assertTrue(self._device.open())

        # You're allowed to open a device multiple times.  It's already been
        # opened once above.  Try to open it again and make sure the count is
        # is right.
        self.assertTrue(self._device.open())
        self.assertEqual(self._device.open_count, 2)
        self._device.close()
        self._device.close()


class DeviceCloseTestCase(RequiresDevice):
    def runTest(self):
        self._device.open()
        self.assertTrue(self._device.close())
        self.assertEqual(self._device.open_count, 0)

        # Not allowed to close a device that's already been opened for external
        # access, so test that now.
        self._device.open()
        self._device.begin_external_access()
        self.assertRaises(_ped.IOException, self._device.close)
        self._device.end_external_access()
        self.assertTrue(self._device.close())

        # Test opening a device multiple times and then closing it too many.
        self._device.open()
        self._device.open()
        self.assertEqual(self._device.open_count, 2)
        self._device.close()
        self.assertEqual(self._device.open_count, 1)
        self._device.close()
        self.assertEqual(self._device.open_count, 0)
        self.assertRaises(_ped.IOException, self._device.close)


@unittest.skip("Unimplemented test case.")
class DeviceDestroyTestCase(RequiresDevice):
    def runTest(self):
        # XXX: still broken, need to fix destroy function in pydevice.c
        # self.assertEqual(self._device.destroy(), None)
        self.fail("Unimplemented test case.")


class DeviceCacheRemoveTestCase(RequiresDevice):
    def runTest(self):
        self.assertEqual(self._device.cache_remove(), None)


class DeviceBeginExternalAccessTestCase(RequiresDevice):
    def runTest(self):
        # First test external access on a device that's not open.
        self.assertEqual(self._device.external_mode, 0)
        self.assertTrue(self._device.begin_external_access())
        self.assertEqual(self._device.external_mode, 1)
        self.assertEqual(self._device.open_count, 0)

        # Now stop external access, open the device, and re-test.
        self._device.end_external_access()
        self._device.open()
        self.assertEqual(self._device.open_count, 1)
        self.assertTrue(self._device.begin_external_access())
        self.assertEqual(self._device.open_count, 1)
        self._device.end_external_access()
        self._device.close()


class DeviceEndExternalAccessTestCase(RequiresDevice):
    def runTest(self):
        # Attempt to end external access on a device that never had it begun.
        self.assertRaises(_ped.IOException, self._device.end_external_access)

        # Now test external access on a device that's not open.
        self._device.begin_external_access()
        self.assertEqual(self._device.external_mode, 1)
        self.assertEqual(self._device.open_count, 0)
        self.assertTrue(self._device.end_external_access())
        self.assertEqual(self._device.external_mode, 0)
        self.assertEqual(self._device.open_count, 0)

        # Now on a device that's open.
        self._device.open()
        self._device.begin_external_access()
        self.assertEqual(self._device.external_mode, 1)
        self.assertEqual(self._device.open_count, 1)
        self.assertTrue(self._device.end_external_access())
        self.assertEqual(self._device.external_mode, 0)
        self.assertEqual(self._device.open_count, 1)
        self._device.close()


@unittest.skip("Unimplemented test case.")
class DeviceReadTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class DeviceWriteTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


class DeviceSyncTestCase(RequiresDevice):
    def runTest(self):
        # Can't sync a device that's not open or is in external mode.
        self.assertRaises(_ped.IOException, self._device.sync)

        self._device.open()
        self._device.begin_external_access()
        self.assertRaises(_ped.IOException, self._device.sync)

        # But this call should work.
        self._device.end_external_access()
        self.assertTrue(self._device.sync())
        self._device.close()


class DeviceSyncFastTestCase(RequiresDevice):
    def runTest(self):
        # Can't sync a device that's not open or is in external mode.
        self.assertRaises(_ped.IOException, self._device.sync_fast)

        self._device.open()
        self._device.begin_external_access()
        self.assertRaises(_ped.IOException, self._device.sync_fast)

        # But this call should work.
        self._device.end_external_access()
        self.assertTrue(self._device.sync_fast())
        self._device.close()


class DeviceCheckTestCase(RequiresDevice):
    def runTest(self):
        self._device.open()
        self.assertEqual(self._device.check(0, 20), 20)
        self._device.close()


class DeviceGetConstraintTestCase(RequiresDevice):
    def runTest(self):
        # XXX: This test case would be a lot more useful testing on real
        # hardware with unusual sector sizes.
        self.assertIsInstance(self._device.get_constraint(), _ped.Constraint)


class DeviceGetMinimalAlignedConstraintTestCase(RequiresDevice):
    def runTest(self):
        # XXX: This test case would be a lot more useful testing on real
        # hardware with unusual sector sizes.
        constraint = self._device.get_minimal_aligned_constraint()
        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertEqual(constraint.start_align.offset, 0)
        self.assertEqual(constraint.start_align.grain_size, 1)
        self.assertEqual(constraint.end_align.offset, 0)
        self.assertEqual(constraint.end_align.grain_size, 1)


class DeviceGetOptimalAlignedConstraintTestCase(RequiresDevice):
    def runTest(self):
        # XXX: This test case would be a lot more useful testing on real
        # hardware with unusual sector sizes.
        constraint = self._device.get_minimal_aligned_constraint()
        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertEqual(constraint.start_align.offset, 0)
        self.assertEqual(constraint.start_align.grain_size, 1)
        self.assertEqual(constraint.end_align.offset, 0)
        self.assertEqual(constraint.end_align.grain_size, 1)


class DeviceGetMinimumAlignmentTestCase(RequiresDevice):
    def runTest(self):
        # XXX: This test case would be a lot more useful testing on real
        # hardware with unusual sector sizes.
        alignment = self._device.get_minimum_alignment()
        self.assertIsInstance(alignment, _ped.Alignment)
        self.assertEqual(alignment.grain_size, 1)
        self.assertEqual(alignment.offset, 0)


class DeviceGetOptimumAlignmentTestCase(RequiresDevice):
    def runTest(self):
        # XXX: This test case would be a lot more useful testing on real
        # hardware with unusual sector sizes.
        alignment = self._device.get_optimum_alignment()
        self.assertIsInstance(alignment, _ped.Alignment)
        self.assertEqual(alignment.grain_size, 2048)
        self.assertEqual(alignment.offset, 0)


class UnitFormatCustomByteTestCase(RequiresDevice):
    def setUp(self):
        super().setUp()
        pr = "%f" % (47.0 / self._device.unit_get_size(_ped.UNIT_PERCENT),)
        self.pairs = [
            (
                _ped.UNIT_SECTOR,
                "0s",
            ),
            (
                _ped.UNIT_BYTE,
                "47B",
            ),
            (
                _ped.UNIT_KILOBYTE,
                "0.05kB",
            ),
            (
                _ped.UNIT_MEGABYTE,
                "0.00MB",
            ),
            (
                _ped.UNIT_GIGABYTE,
                "0.00GB",
            ),
            (
                _ped.UNIT_TERABYTE,
                "0.00TB",
            ),
            (
                _ped.UNIT_COMPACT,
                "47.0B",
            ),
            (
                _ped.UNIT_CYLINDER,
                "0cyl",
            ),
            (
                _ped.UNIT_CHS,
                "0,0,0",
            ),
            (
                _ped.UNIT_PERCENT,
                pr[:4] + "%",
            ),
            (
                _ped.UNIT_KIBIBYTE,
                "0.05kiB",
            ),
            (
                _ped.UNIT_MEBIBYTE,
                "0.00MiB",
            ),
            (
                _ped.UNIT_GIBIBYTE,
                "0.00GiB",
            ),
            (
                _ped.UNIT_TEBIBYTE,
                "0.00TiB",
            ),
        ]

    def runTest(self):
        for (
            unit,
            expected,
        ) in self.pairs:
            self.assertEqual(self._device.unit_format_custom_byte(47, unit), expected)


class UnitFormatByteTestCase(RequiresDevice):
    def setUp(self):
        super().setUp()
        pr = "%f" % (47.0 / self._device.unit_get_size(_ped.UNIT_PERCENT),)
        self._initialDefault = _ped.unit_get_default()
        self.pairs = [
            (
                _ped.UNIT_SECTOR,
                "0s",
            ),
            (
                _ped.UNIT_BYTE,
                "47B",
            ),
            (
                _ped.UNIT_KILOBYTE,
                "0.05kB",
            ),
            (
                _ped.UNIT_MEGABYTE,
                "0.00MB",
            ),
            (
                _ped.UNIT_GIGABYTE,
                "0.00GB",
            ),
            (
                _ped.UNIT_TERABYTE,
                "0.00TB",
            ),
            (
                _ped.UNIT_COMPACT,
                "47.0B",
            ),
            (
                _ped.UNIT_CYLINDER,
                "0cyl",
            ),
            (
                _ped.UNIT_CHS,
                "0,0,0",
            ),
            (
                _ped.UNIT_PERCENT,
                pr[:4] + "%",
            ),
            (
                _ped.UNIT_KIBIBYTE,
                "0.05kiB",
            ),
            (
                _ped.UNIT_MEBIBYTE,
                "0.00MiB",
            ),
            (
                _ped.UNIT_GIBIBYTE,
                "0.00GiB",
            ),
            (
                _ped.UNIT_TEBIBYTE,
                "0.00TiB",
            ),
        ]

    def runTest(self):
        for (unit, expected) in self.pairs:
            _ped.unit_set_default(unit)
            result = self._device.unit_format_byte(47)
            self.assertEqual(result, expected)

    def tearDown(self):
        _ped.unit_set_default(self._initialDefault)


class UnitFormatCustomTestCase(RequiresDevice):
    def setUp(self):
        super().setUp()
        sector_size = self._device.sector_size
        size = self._device.unit_get_size(_ped.UNIT_PERCENT)
        pr = "%f" % ((47.0 * sector_size) / size,)
        self.pairs = [
            (
                _ped.UNIT_SECTOR,
                "47s",
            ),
            (
                _ped.UNIT_BYTE,
                "24064B",
            ),
            (
                _ped.UNIT_KILOBYTE,
                "24.1kB",
            ),
            (
                _ped.UNIT_MEGABYTE,
                "0.02MB",
            ),
            (
                _ped.UNIT_GIGABYTE,
                "0.00GB",
            ),
            (
                _ped.UNIT_TERABYTE,
                "0.00TB",
            ),
            (
                _ped.UNIT_COMPACT,
                "24.1kB",
            ),
            (
                _ped.UNIT_CYLINDER,
                "0cyl",
            ),
            (
                _ped.UNIT_CHS,
                "0,1,15",
            ),
            (
                _ped.UNIT_PERCENT,
                pr[:4] + "%",
            ),
            (
                _ped.UNIT_KIBIBYTE,
                "23.5kiB",
            ),
            (
                _ped.UNIT_MEBIBYTE,
                "0.02MiB",
            ),
            (
                _ped.UNIT_GIBIBYTE,
                "0.00GiB",
            ),
            (
                _ped.UNIT_TEBIBYTE,
                "0.00TiB",
            ),
        ]

    def runTest(self):
        for (unit, expected) in self.pairs:
            result = self._device.unit_format_custom(47, unit)
            self.assertEqual(result, expected)


class UnitFormatTestCase(RequiresDevice):
    def setUp(self):
        super().setUp()
        sector_size = self._device.sector_size
        size = self._device.unit_get_size(_ped.UNIT_PERCENT)
        pr = "%f" % ((47.0 * sector_size) / size,)
        self._initialDefault = _ped.unit_get_default()
        self.pairs = [
            (
                _ped.UNIT_SECTOR,
                "47s",
            ),
            (
                _ped.UNIT_BYTE,
                "24064B",
            ),
            (
                _ped.UNIT_KILOBYTE,
                "24.1kB",
            ),
            (
                _ped.UNIT_MEGABYTE,
                "0.02MB",
            ),
            (
                _ped.UNIT_GIGABYTE,
                "0.00GB",
            ),
            (
                _ped.UNIT_TERABYTE,
                "0.00TB",
            ),
            (
                _ped.UNIT_COMPACT,
                "24.1kB",
            ),
            (
                _ped.UNIT_CYLINDER,
                "0cyl",
            ),
            (
                _ped.UNIT_CHS,
                "0,1,15",
            ),
            (
                _ped.UNIT_PERCENT,
                pr[:4] + "%",
            ),
            (
                _ped.UNIT_KIBIBYTE,
                "23.5kiB",
            ),
            (
                _ped.UNIT_MEBIBYTE,
                "0.02MiB",
            ),
            (
                _ped.UNIT_GIBIBYTE,
                "0.00GiB",
            ),
            (
                _ped.UNIT_TEBIBYTE,
                "0.00TiB",
            ),
        ]

    def runTest(self):
        for (unit, expected) in self.pairs:
            _ped.unit_set_default(unit)
            result = self._device.unit_format(47)
            self.assertEqual(result, expected)

    def tearDown(self):
        _ped.unit_set_default(self._initialDefault)


@unittest.skip("Unimplemented test case.")
class UnitParseTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class UnitParseCustomTestCase(unittest.TestCase):
    # TODO
    def runTest(self):
        self.fail("Unimplemented test case.")


class DeviceStrTestCase(RequiresDevice):
    def runTest(self):
        expected = (
            "_ped.Device instance --\n  model: %s  path: %s  type: %d\n  sector_size: %d  phys_sector_size: %d\n  length: %d  open_count: %d  read_only: %d\n  external_mode: %d  dirty: %d  boot_dirty: %d\n  host: %d  did: %d\n  hw_geom: %s  bios_geom: %s"
            % (
                self._device.model,
                self._device.path,
                self._device.type,
                self._device.sector_size,
                self._device.phys_sector_size,
                self._device.length,
                self._device.open_count,
                self._device.read_only,
                self._device.external_mode,
                self._device.dirty,
                self._device.boot_dirty,
                self._device.host,
                self._device.did,
                repr(self._device.hw_geom),
                repr(self._device.bios_geom),
            )
        )
        self.assertEqual(str(self._device), expected)



#
# Test cases for the methods in the _ped module itself - just the pyunit
# and pynatmath files.
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import os
import _ped
import unittest

from tests.baseclass import BuildList, RequiresDevice, RequiresFileSystem

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class PartitionFlagGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in [
            "PARTITION_BOOT",
            "PARTITION_ROOT",
            "PARTITION_SWAP",
            "PARTITION_HIDDEN",
            "PARTITION_RAID",
            "PARTITION_LVM",
            "PARTITION_LBA",
            "PARTITION_HPSERVICE",
            "PARTITION_PALO",
            "PARTITION_PREP",
            "PARTITION_MSFT_RESERVED",
            "PARTITION_APPLE_TV_RECOVERY",
            "PARTITION_BIOS_GRUB",
            "PARTITION_DIAG",
            "PARTITION_MSFT_DATA",
            "PARTITION_IRST",
            "PARTITION_ESP",
            "PARTITION_NONFS",
            "PARTITION_CHROMEOS_KERNEL",
            "PARTITION_CHROMEOS_KERNEL",
        ]:
            if not hasattr(_ped, f):
                continue
            attr = getattr(_ped, f)
            self.assertNotEqual(
                _ped.partition_flag_get_name(attr),
                "",
                "Could not get name for flag _ped.%s" % f,
            )

        self.assertRaises(ValueError, _ped.partition_flag_get_name, -1)
        self.assertRaises(_ped.PartedException, _ped.partition_flag_get_name, 1000)


class PartitionFlagGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in [
            "boot",
            "root",
            "swap",
            "hidden",
            "raid",
            "lvm",
            "lba",
            "hp-service",
            "palo",
            "prep",
            "msftres",
            "bios_grub",
            "msftdata",
            "irst",
            "esp",
            "nonfs",
        ]:
            self.assertNotEqual(
                _ped.partition_flag_get_by_name(f), "", "Could not get flag %s" % f
            )

        self.assertEqual(_ped.partition_flag_get_by_name("nosuchflag"), 0)


class PartitionFlagNextTestCase(unittest.TestCase):
    def runTest(self):
        # We should get TypeError when the parameter is invalid
        self.assertRaises(TypeError, _ped.partition_flag_next)
        self.assertRaises(TypeError, _ped.partition_flag_next, "blah")

        # First flag is 0, keep getting flags until we loop back around
        # to zero.  Make sure each flag we get is an integer.
        flag = _ped.partition_flag_next(0)
        self.assertEqual(type(flag).__name__, "int")

        while True:
            flag = _ped.partition_flag_next(flag)
            if not flag:
                break
            self.assertEqual(type(flag).__name__, "int")


class DiskFlagGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in [_ped.DISK_CYLINDER_ALIGNMENT]:
            self.assertNotEqual(
                _ped.disk_flag_get_name(f), "", "Could not get name for flag %s" % f
            )

        self.assertRaises(ValueError, _ped.disk_flag_get_name, -1)
        self.assertRaises(_ped.PartedException, _ped.disk_flag_get_name, 1000)


class DiskFlagGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        for f in ["cylinder_alignment"]:
            self.assertNotEqual(
                _ped.disk_flag_get_by_name(f), 0, "Could not get flag %s" % f
            )

        self.assertEqual(_ped.disk_flag_get_by_name("nosuchflag"), 0)


class DiskFlagNextTestCase(unittest.TestCase):
    def runTest(self):
        # We should get TypeError when the parameter is invalid
        self.assertRaises(TypeError, _ped.disk_flag_next)
        self.assertRaises(TypeError, _ped.disk_flag_next, "blah")

        # First flag is 0, keep getting flags until we loop back around
        # to zero.  Make sure each flag we get is an integer.
        flag = _ped.disk_flag_next(0)
        self.assertEqual(type(flag).__name__, "int")

        while True:
            flag = _ped.disk_flag_next(flag)
            if not flag:
                break
            self.assertEqual(type(flag).__name__, "int")


class ConstraintNewFromMinMaxTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_min_max, None)

        # min is required to be within max, so test various combinations of
        # that not being the case.
        self.assertRaises(
            _ped.CreateException,
            _ped.constraint_new_from_min_max,
            _ped.Geometry(self._device, 0, 10),
            _ped.Geometry(self._device, 15, 25),
        )
        self.assertRaises(
            _ped.CreateException,
            _ped.constraint_new_from_min_max,
            _ped.Geometry(self._device, 10, 20),
            _ped.Geometry(self._device, 15, 25),
        )

        # Now test a correct call.
        minimum = _ped.Geometry(self._device, 10, 20)
        maximum = _ped.Geometry(self._device, 0, 30)
        constraint = _ped.constraint_new_from_min_max(minimum, maximum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 11, 20)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 0, 30)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 0, 35)))


class ConstraintNewFromMinTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_min, None)

        minimum = _ped.Geometry(self._device, 10, 20)
        constraint = _ped.constraint_new_from_min(minimum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 11, 19)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 15, 25)))


class ConstraintNewFromMaxTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_new_from_max, None)

        maximum = _ped.Geometry(self._device, 10, 20)
        constraint = _ped.constraint_new_from_max(maximum)

        self.assertIsInstance(constraint, _ped.Constraint)
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 10, 20)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 5, 25)))
        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 11, 19)))
        self.assertFalse(constraint.is_solution(_ped.Geometry(self._device, 15, 25)))


class ConstraintAnyTestCase(RequiresDevice):
    def runTest(self):
        self.assertRaises(TypeError, _ped.constraint_any, None)

        constraint = _ped.constraint_any(self._device)
        self.assertIsInstance(constraint, _ped.Constraint)

        for testGeom in [
            _ped.Geometry(self._device, 0, 5),
            _ped.Geometry(self._device, 10, 25),
            _ped.Geometry(self._device, 0, 100),
        ]:
            self.assertTrue(constraint.is_solution(testGeom))


class ConstraintExactTestCase(RequiresDevice):
    def runTest(self):
        geom = _ped.Geometry(self._device, 0, 100)

        self.assertRaises(TypeError, _ped.constraint_exact, None)

        constraint = _ped.constraint_exact(geom)
        self.assertIsInstance(constraint, _ped.Constraint)

        for testGeom in [
            _ped.Geometry(self._device, 1, 100),
            _ped.Geometry(self._device, 0, 99),
            _ped.Geometry(self._device, 10, 20),
            _ped.Geometry(self._device, 50, 101),
        ]:
            self.assertFalse(constraint.is_solution(testGeom))

        self.assertTrue(constraint.is_solution(_ped.Geometry(self._device, 0, 100)))


class DeviceGetTestCase(RequiresDevice):
    def runTest(self):
        # Try getting the device we just made.
        self.assertIsInstance(_ped.device_get(self.path), _ped.Device)

        # Try getting a device that doesn't exist.
        self.assertRaises(_ped.IOException, _ped.device_get, "/blah/whatever")
        self.assertRaises(_ped.IOException, _ped.device_get, "")
        self.assertRaises(_ped.DeviceException, _ped.device_get, None)


@unittest.skipUnless(os.geteuid() == 0, "Requires root")
@unittest.skipUnless("container" not in os.environ, "Running in a container")
class DeviceGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        # Make sure there are some devices in the system first and then
        # make a list out of them.  That's easier to work with.
        _ped.device_probe_all()
        lst = self.getDeviceList(_ped.device_get_next)

        # Now the test cases.
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.device_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.Device)

        self.assertRaises(IndexError, _ped.device_get_next, lst[-1])


class DeviceProbeAllTestCase(RequiresDevice, BuildList):
    def runTest(self):
        # Since we inherit from RequiresDevice, we can test that the temp
        # device we created is in the results list.  I can't really think of
        # any other way to test this method except by getting a list of devices
        # via some other mechanism and comparing that to the device_probe_all
        # results.
        _ped.device_probe_all()
        lst = self.getDeviceList(_ped.device_get_next)
        prefix = (
            self.path[: self.path.index(self.temp_prefix) - 1] + "/" + self.temp_prefix
        )

        self.assertGreater(len(lst), 0)
        self.assertGreater(len([e for e in lst if e.path.startswith(prefix)]), 0)


class DiskTypeGetTestCase(unittest.TestCase):
    def runTest(self):
        for d in [
            "aix",
            "amiga",
            "bsd",
            "dvh",
            "gpt",
            "loop",
            "mac",
            "msdos",
            "pc98",
            "sun",
        ]:
            t = _ped.disk_type_get(d)
            self.assertIsInstance(t, _ped.DiskType)
            self.assertEqual(t.name, d)

        self.assertRaises(_ped.UnknownTypeException, _ped.disk_type_get, "nosuch")


class DiskTypeGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        lst = self.getDeviceList(_ped.disk_type_get_next)
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.device_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.DiskType)

        self.assertRaises(IndexError, _ped.disk_type_get_next, lst[-1])


class FileSystemProbeTestCase(RequiresFileSystem):
    def runTest(self):
        ty = _ped.file_system_probe(self._geometry)

        for name in self._fileSystemType.keys():
            if name == "ext2":
                self.assertEqual(ty.name, name)
            else:
                self.assertNotEqual(ty.name, name)


class FileSystemProbeSpecificTestCase(RequiresFileSystem):
    def runTest(self):
        for (
            name,
            ty,
        ) in self._fileSystemType.items():
            if name == "ext2":
                result = _ped.file_system_probe_specific(ty, self._geometry)

                # XXX: this should work
                # we're getting
                #     ValueError: object comparing to must be a _ped.Geometry
                # at runtime.  works fine in pdb.
                # self.assertEqual(result, self._geometry)

                self.assertIsInstance(result, _ped.Geometry)
                self.assertEqual(result.start, self._geometry.start)
                self.assertLessEqual(result.end, self._geometry.end)
                self.assertLessEqual(result.length, self._geometry.length)
                self.assertEqual(result.dev, self._device)
            else:
                result = _ped.file_system_probe_specific(ty, self._geometry)
                self.assertEqual(result, None)


class FileSystemTypeGetTestCase(unittest.TestCase):
    def runTest(self):
        for f in [
            "affs0",
            "amufs",
            "apfs1",
            "asfs",
            "btrfs",
            "ext2",
            "ext3",
            "ext4",
            "fat16",
            "fat32",
            "hfs",
            "hfs+",
            "hfsx",
            "hp-ufs",
            "jfs",
            "linux-swap",
            "ntfs",
            "reiserfs",
            "sun-ufs",
            "xfs",
        ]:
            # may be missing some filesystem types depending on the parted
            # build on the test system
            try:
                t = _ped.file_system_type_get(f)
                self.assertIsInstance(
                    t, _ped.FileSystemType, "Could not get fs type %s" % t
                )
            except _ped.UnknownTypeException:
                pass

        self.assertRaises(
            _ped.UnknownTypeException, _ped.file_system_type_get, "nosuch"
        )


class FileSystemTypeGetNextTestCase(unittest.TestCase, BuildList):
    def runTest(self):
        lst = self.getDeviceList(_ped.file_system_type_get_next)
        self.assertGreater(len(lst), 0)
        self.assertRaises(TypeError, _ped.file_system_type_get_next, None)

        for ele in lst:
            self.assertIsInstance(ele, _ped.FileSystemType)

        self.assertRaises(IndexError, _ped.file_system_type_get_next, lst[-1])


class PartitionTypeGetNameTestCase(unittest.TestCase):
    def runTest(self):
        for t in [
            _ped.PARTITION_METADATA,
            _ped.PARTITION_FREESPACE,
            _ped.PARTITION_EXTENDED,
            _ped.PARTITION_LOGICAL,
        ]:
            self.assertNotEqual(
                _ped.partition_type_get_name(t),
                "",
                "Could not get name for flag %s" % t,
            )


class UnitSetDefaultTestCase(unittest.TestCase):
    def setUp(self):
        self._initialDefault = _ped.unit_get_default()

    def tearDown(self):
        _ped.unit_set_default(self._initialDefault)

    def runTest(self):
        for v in [
            _ped.UNIT_BYTE,
            _ped.UNIT_CHS,
            _ped.UNIT_COMPACT,
            _ped.UNIT_CYLINDER,
            _ped.UNIT_GIBIBYTE,
            _ped.UNIT_GIGABYTE,
            _ped.UNIT_KIBIBYTE,
            _ped.UNIT_KILOBYTE,
            _ped.UNIT_MEBIBYTE,
            _ped.UNIT_MEGABYTE,
            _ped.UNIT_PERCENT,
            _ped.UNIT_SECTOR,
            _ped.UNIT_TEBIBYTE,
            _ped.UNIT_TERABYTE,
        ]:
            _ped.unit_set_default(v)
            self.assertEqual(
                _ped.unit_get_default(), v, "Could not set unit default to %s" % v
            )

        self.assertRaises(ValueError, _ped.unit_set_default, -1)
        self.assertRaises(ValueError, _ped.unit_set_default, 1000)


class UnitGetDefaultTestCase(unittest.TestCase):
    def runTest(self):
        self.assertGreaterEqual(_ped.unit_get_default(), 0)


class UnitGetSizeTestCase(RequiresDevice):
    def runTest(self):
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_SECTOR), 512)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_BYTE), 1)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_KILOBYTE), 1000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_MEGABYTE), 1000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_GIGABYTE), 1000000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_TERABYTE), 1000000000000)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_KIBIBYTE), 1024)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_MEBIBYTE), 1048576)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_GIBIBYTE), 1073741824)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_TEBIBYTE), 1099511627776)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_CYLINDER), 65536)
        self.assertEqual(self._device.unit_get_size(_ped.UNIT_CHS), 512)
        self.assertEqual(
            self._device.unit_get_size(_ped.UNIT_PERCENT),
            self._device.length * self._device.sector_size // 100,
        )
        self.assertRaises(ValueError, self._device.unit_get_size, _ped.UNIT_COMPACT)


class UnitGetNameTestCase(unittest.TestCase):
    def runTest(self):
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_BYTE), "B")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_CHS), "chs")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_COMPACT), "compact")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_CYLINDER), "cyl")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_GIBIBYTE), "GiB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_GIGABYTE), "GB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_KIBIBYTE), "kiB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_KILOBYTE), "kB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_MEBIBYTE), "MiB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_MEGABYTE), "MB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_PERCENT), "%")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_SECTOR), "s")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_TEBIBYTE), "TiB")
        self.assertEqual(_ped.unit_get_name(_ped.UNIT_TERABYTE), "TB")

        self.assertRaises(ValueError, _ped.unit_get_name, -1)
        self.assertRaises(ValueError, _ped.unit_get_name, 1000)


class UnitGetByNameTestCase(unittest.TestCase):
    def runTest(self):
        self.assertEqual(_ped.unit_get_by_name("B"), _ped.UNIT_BYTE)
        self.assertEqual(_ped.unit_get_by_name("chs"), _ped.UNIT_CHS)
        self.assertEqual(_ped.unit_get_by_name("compact"), _ped.UNIT_COMPACT)
        self.assertEqual(_ped.unit_get_by_name("cyl"), _ped.UNIT_CYLINDER)
        self.assertEqual(_ped.unit_get_by_name("GiB"), _ped.UNIT_GIBIBYTE)
        self.assertEqual(_ped.unit_get_by_name("GB"), _ped.UNIT_GIGABYTE)
        self.assertEqual(_ped.unit_get_by_name("kiB"), _ped.UNIT_KIBIBYTE)
        self.assertEqual(_ped.unit_get_by_name("kB"), _ped.UNIT_KILOBYTE)
        self.assertEqual(_ped.unit_get_by_name("MiB"), _ped.UNIT_MEBIBYTE)
        self.assertEqual(_ped.unit_get_by_name("MB"), _ped.UNIT_MEGABYTE)
        self.assertEqual(_ped.unit_get_by_name("%"), _ped.UNIT_PERCENT)
        self.assertEqual(_ped.unit_get_by_name("s"), _ped.UNIT_SECTOR)
        self.assertEqual(_ped.unit_get_by_name("TiB"), _ped.UNIT_TEBIBYTE)
        self.assertEqual(_ped.unit_get_by_name("TB"), _ped.UNIT_TERABYTE)

        self.assertRaises(_ped.UnknownTypeException, _ped.unit_get_by_name, "blargle")



#
# Test cases for the methods in the parted.filesystem module itself
#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import unittest

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
@unittest.skip("Unimplemented test case.")
class FileSystemNewTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class FileSystemGetSetTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class FileSystemGetPedFileSystemTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")


@unittest.skip("Unimplemented test case.")
class FileSystemStrTestCase(unittest.TestCase):
    def runTest(self):
        # TODO
        self.fail("Unimplemented test case.")



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import parted
import os
import tempfile
import unittest

# Base class for any test case that requires a temp device node
class RequiresDeviceNode(unittest.TestCase):
    def setUp(self):
        super().setUp()

        self.temp_prefix = "temp-device-"
        (self.fd, self.path) = tempfile.mkstemp(prefix=self.temp_prefix)
        self.f = os.fdopen(self.fd)
        self.f.seek(140000)
        os.write(self.fd, b"0")

    def tearDown(self):
        os.close(self.fd)

        if self.path and os.path.exists(self.path):
            os.unlink(self.path)

        self.fd = None
        self.path = None
        self.temp_prefix = None


# Base class for any test case that requires a _ped.Device or parted.Device
# object first.
class RequiresDevice(RequiresDeviceNode):
    def setUp(self):
        super().setUp()
        self.addCleanup(self.removeDevice)
        self._device = _ped.device_get(self.path)
        self.device = parted.getDevice(self.path)

    def removeDevice(self):
        self.device = None
        self._device = None


# Base class for any test case that requires a filesystem on a device.
class RequiresFileSystem(unittest.TestCase):
    def setUp(self):
        super().setUp()

        self._fileSystemType = {}
        ty = _ped.file_system_type_get_next()
        self._fileSystemType[ty.name] = ty

        while True:
            try:
                ty = _ped.file_system_type_get_next(ty)
                self._fileSystemType[ty.name] = ty
            except (IndexError, TypeError, _ped.UnknownTypeException):
                break

        self.temp_prefix = "temp-device-"
        (
            self.fd,
            self.path,
        ) = tempfile.mkstemp(prefix=self.temp_prefix)
        self.f = os.fdopen(self.fd)
        self.f.seek(140000)
        os.write(self.fd, b"0")
        self.f.close()

        os.system("mke2fs -F -q %s" % (self.path,))

        self._device = _ped.device_get(self.path)
        self._geometry = _ped.Geometry(self._device, 0, self._device.length - 1)

    def tearDown(self):
        if self.path and os.path.exists(self.path):
            os.unlink(self.path)

        self.mountpoint = None


# Base class for certain alignment tests that require a _ped.Device
class RequiresDeviceAlignment(RequiresDevice):
    def roundDownTo(self, sector, grain_size):
        if sector < 0:
            shift = sector % grain_size + grain_size
        else:
            shift = sector % grain_size

        return sector - shift

    def roundUpTo(self, sector, grain_size):
        if sector % grain_size:
            return self.roundDownTo(sector, grain_size) + grain_size
        else:
            return sector

    def closestInsideGeometry(self, alignment, geometry, sector):
        if alignment.grain_size == 0:
            if alignment.is_aligned(geometry, sector) and (
                (geometry is None) or geometry.test_sector_inside(sector)
            ):
                return sector
            else:
                return -1

        if sector < geometry.start:
            sector += self.roundUpTo(geometry.start - sector, alignment.grain_size)

        if sector > geometry.end:
            sector -= self.roundUpTo(sector - geometry.end, alignment.grain_size)

        if not geometry.test_sector_inside(sector):
            return -1

        return sector

    def closest(self, sector, a, b):
        if a == -1:
            return b

        if b == -1:
            return a

        if abs(sector - a) < abs(sector - b):
            return a
        else:
            return b


# Base class for any test case that requires a labeled device
class RequiresLabeledDevice(RequiresDevice):
    def setUp(self):
        super().setUp()
        os.system("parted -s %s mklabel msdos" % (self.path,))


# Base class for any test case that requires a _ped.Disk or parted.Disk.
class RequiresDisk(RequiresDevice):
    def setUp(self):
        super().setUp()
        self._disk = _ped.disk_new_fresh(self._device, _ped.disk_type_get("msdos"))
        self.disk = parted.Disk(PedDisk=self._disk)

    def reopen(self):
        self._disk = _ped.disk_new(self._device)
        self.disk = parted.Disk(PedDisk=self._disk)


# Base class for any test case that requires a GPT-labeled _ped.Disk or parted.Disk.
class RequiresGPTDisk(RequiresDevice):
    def setUp(self):
        super().setUp()
        self._disk = _ped.disk_new_fresh(self._device, _ped.disk_type_get("gpt"))
        self.disk = parted.Disk(PedDisk=self._disk)

    def reopen(self):
        self._disk = _ped.disk_new(self._device)
        self.disk = parted.Disk(PedDisk=self._disk)


# Base class for any test case that requires a filesystem made and mounted.
class RequiresMount(RequiresDevice):
    def setUp(self):
        super().setUp()
        self.addCleanup(self.removeMountpoint)
        self.mountpoint = None

    def mkfs(self):
        os.system("mkfs.ext2 -F -q %s" % self.path)

    def doMount(self):
        self.mountpoint = tempfile.mkdtemp()
        os.system("mount -o loop %s %s" % (self.path, self.mountpoint))

    def removeMountpoint(self):
        if self.mountpoint and os.path.exists(self.mountpoint):
            os.system("umount %s" % self.mountpoint)
            os.rmdir(self.mountpoint)


# Base class for any test case that requires a _ped.Partition.
class RequiresPartition(RequiresDisk):
    def setUp(self):
        super().setUp()
        self._part = _ped.Partition(
            disk=self._disk,
            type=_ped.PARTITION_NORMAL,
            start=1,
            end=100,
            fs_type=_ped.file_system_type_get("ext2"),
        )

    def reopen(self):
        super().reopen()
        self._part = self._disk.next_partition(self._disk.next_partition())


# Base class for any test case that requires a _ped.Partition on GPT disk.
class RequiresGPTPartition(RequiresGPTDisk):
    def setUp(self):
        super().setUp()
        self._part = _ped.Partition(
            disk=self._disk,
            type=_ped.PARTITION_NORMAL,
            start=0,
            end=100,
            fs_type=_ped.file_system_type_get("ext2"),
        )

    def reopen(self):
        super().reopen()
        self._part = self._disk.next_partition()


# Base class for any test case that requires a hash table of all
# _ped.DiskType objects available
class RequiresDiskTypes(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.disktype = {}
        ty = _ped.disk_type_get_next()
        self.disktype[ty.name] = ty

        while True:
            try:
                ty = _ped.disk_type_get_next(ty)
                self.disktype[ty.name] = ty
            except (IndexError, TypeError, _ped.UnknownTypeException):
                break


# Base class for any test case that requires a list being built via successive
# calls of some function.  The function must raise IndexError when there's no
# more output to add to the return list.  This class is most useful for all
# those _get_next methods.
class BuildList:
    def getDeviceList(self, func):
        lst = []
        prev = None

        while True:
            try:
                if not prev:
                    prev = func()
                else:
                    prev = func(prev)

                lst.append(prev)
            except IndexError:
                break

        return lst



#
# Copyright The pyparted Project Authors
# SPDX-License-Identifier: GPL-2.0-or-later
#

import _ped
import sys
import unittest
from tests.baseclass import RequiresDiskTypes

# One class per method, multiple tests per class.  For these simple methods,
# that seems like good organization.  More complicated methods may require
# multiple classes and their own test suite.
class DiskTypeNewTestCase(unittest.TestCase):
    def runTest(self):
        # You're not allowed to create a new DiskType object by hand.
        self.assertRaises(TypeError, _ped.DiskType)


class DiskTypeGetSetTestCase(RequiresDiskTypes):
    def runTest(self):
        # All attributes are read-only.
        for name in self.disktype.keys():
            t = self.disktype[name]

            self.assertRaises(AttributeError, setattr, t, "name", "fakename")
            self.assertRaises(AttributeError, setattr, t, "features", 47)

            if sys.version_info >= (3,):
                bigint = int
                uni = str
            else:
                bigint = long  # pylint: disable=undefined-variable
                uni = unicode  # pylint: disable=undefined-variable
            self.assertIsInstance(t.name, uni)
            self.assertEqual(t.name, name)
            self.assertIsInstance(t.features, bigint)


class DiskTypeCheckFeatureTestCase(RequiresDiskTypes):
    def runTest(self):
        # The following types have no features [that libparted supports]
        for name in ["aix", "sun", "bsd", "loop"]:
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertFalse(
                self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME)
            )

        # The following types support DISK_TYPE_EXTENDED
        for name in ["msdos"]:
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertFalse(
                self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME)
            )

            if hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_ID"):
                self.assertTrue(
                    self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_TYPE_ID)
                )

            if hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_UUID"):
                self.assertFalse(
                    self.disktype[name].check_feature(
                        _ped.DISK_TYPE_PARTITION_TYPE_UUID
                    )
                )

            if hasattr(_ped, "DISK_TYPE_DISK_UUID"):
                self.assertFalse(
                    self.disktype[name].check_feature(_ped.DISK_TYPE_DISK_UUID)
                )

            if hasattr(_ped, "DISK_TYPE_PARTITION_UUID"):
                self.assertFalse(
                    self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_UUID)
                )

        # The following types support DISK_TYPE_PARTITION_NAME
        for name in ["amiga", "gpt", "mac", "pc98"]:
            self.assertFalse(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertTrue(
                self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME)
            )

        # The following types support DISK_TYPE_PARTITION_TYPE_UUID
        for name in ["gpt"]:
            if hasattr(_ped, "DISK_TYPE_PARTITION_TYPE_UUID"):
                self.assertTrue(
                    self.disktype[name].check_feature(
                        _ped.DISK_TYPE_PARTITION_TYPE_UUID
                    )
                )

            if hasattr(_ped, "DISK_TYPE_DISK_UUID"):
                self.assertTrue(
                    self.disktype[name].check_feature(_ped.DISK_TYPE_DISK_UUID)
                )

            if hasattr(_ped, "DISK_TYPE_PARTITION_UUID"):
                self.assertTrue(
                    self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_UUID)
                )

        # The following types support all features
        for name in ["dvh"]:
            self.assertTrue(self.disktype[name].check_feature(_ped.DISK_TYPE_EXTENDED))
            self.assertTrue(
                self.disktype[name].check_feature(_ped.DISK_TYPE_PARTITION_NAME)
            )


class DiskTypeStrTestCase(RequiresDiskTypes):
    def runTest(self):
        types = {
            "msdos": "_ped.DiskType instance --\n  name: msdos  features:",
            "aix": "_ped.DiskType instance --\n  name: aix  features: ",
            "sun": "_ped.DiskType instance --\n  name: sun  features: ",
            "amiga": "_ped.DiskType instance --\n  name: amiga  features: ",
            "gpt": "_ped.DiskType instance --\n  name: gpt  features: ",
            "mac": "_ped.DiskType instance --\n  name: mac  features: ",
            "bsd": "_ped.DiskType instance --\n  name: bsd  features: ",
            "pc98": "_ped.DiskType instance --\n  name: pc98  features: ",
            "loop": "_ped.DiskType instance --\n  name: loop  features: ",
            "dvh": "_ped.DiskType instance --\n  name: dvh  features: ",
        }

        for t, expected in types.items():
            self.assertIn(expected, str(self.disktype[t]))
